from pathlib import Path
from random  import randint

from fileIO import uploadFile
from configs import makeRubinoData, welcome, __version__, __license__, __copyright__

class Rubino:
    def __init__(self, auth:str):
        self.auth = auth
        self.profileID = [i['id'] for i in self.getProfileList().get("data").get("profiles") if i['is_default']][0]
        welcome(f"rubika library version {__version__}\n{__copyright__}\n→ docs : https://rubikalib.github.io\n\nswitching rubino...")

    def addPost(self, file:str, caption:str=None, isMultiFile:bool=None, postType:str="Picture"):
        uresponse:list = self.fileUpload(file, Type=postType)
        return makeRubinoData(self.auth, "addPost", {"caption": caption, "file_id": uresponse[0]['file_id'], "hash_file_receive": uresponse[1], "height": "800", "width": "800", "is_multi_file": isMultiFile, "post_type": postType, "rnd": randint(100000, 999999999), "thumbnail_file_id": uresponse[0]['file_id'], "thumbnail_hash_file_receive": uresponse[1], "profile_id": self.profileID})

    def addStory(self, duration, width, height, storyType="Picture"):
        uresponse:list = self.fileUpload(file, Type=storyType)
        return makeRubinoData(self.auth, "addStory", {"duration": duration, "file_id": uresponse[0]['file_id'], "hash_file_receive": uresponse[1], "height": height, "profile_id": self.profileID, "rnd": randint(100000, 999999999), "story_type": storyType, "thumbnail_file_id": uresponse[0]['file_id'], "thumbnail_hash_file_receive": uresponse[1], "width": width})

    bookmark          = lambda self, postID, postProfileID       : makeRubinoData(self.auth, "postBookmarkAction", {"action_type": "Bookmark","post_id": postID,"post_profile_id": postProfileID,"profile_id": self.profileID})

    createPage        = lambda self, name, username, bio=None    : makeRubinoData(self.auth, "createPage", {"bio": bio,"name": name,"username": username})
    comment           = lambda self, text, postID, postProfileID : makeRubinoData(self.auth, "addComment", {"content": text,"post_id": postID, "post_profile_id": postProfileID, "rnd": str(randint(100000,999999999)) ,"profile_id": self.profileID})

    fileUpload        = lambda self, file, Type="Picture": uploadFile(self, file, rubino=True, rubinoFType=Type, rubinoFRequest=self.requestUploadFile)
    follow            = lambda self, followee_id: makeRubinoData(self.auth, "requestFollow", {"f_type": "Follow", "followee_id": followee_id, "profile_id": self.profileID})
    
    getComments        = lambda self, postID, postProfileID, limit=50, sort="FromMax" : makeRubinoData(self.auth, "getComments", {"equal": False, "limit": limit, "sort": sort, "post_id": postID, "profile_id": self.profileID, "post_profile_id": postProfileID})
    getMyProfileInfo   = lambda self                                                  : makeRubinoData(self.auth, "getMyProfileInfo", {"profile_id": self.profileID})
    getPostByShareLink = lambda self, shareString                                     : makeRubinoData(self.auth, "getPostByShareLink", {"share_string": shareString.replace('https://rubika.ir/post/', ''), "profile_id": self.profileID})
    getProfileList     = lambda self, equal=False, limit=10, sort="FromMax"           : makeRubinoData(self.auth, "getProfileList", {"equal": equal,"limit": limit,"sort": sort})
    getProfilePosts    = lambda self, targetProfileID, limit=51, sort="FromMax"       : makeRubinoData(self.auth, "getProfilePosts", {"equal": False, "limit": limit, "sort": sort, "target_profile_id": targetProfileID, "profile_id": self.profileID})
    getRecentPosts     = lambda self, equal=False, limit=30, sort="FromMax"           : makeRubinoData(self.auth, "getRecentFollowingPosts", {"equal": equal, "limit": limit, "sort": sort, "profile_id": self.profileID})
    getShareLink       = lambda self, postID, postProfileID                           : makeRubinoData(self.auth, "getShareLink", {"post_id": postID, "post_profile_id": postProfileID, "profile_id": self.profileID})
    getStories         = lambda self, targetProfileID, limit=100                      : makeRubinoData(self.auth, "getProfileStories", {"limit": limit, "profile_id": targetProfileID})
    getStoryIds        = lambda self, targetProfileID                                 : makeRubinoData(self.auth, "getStoryIds", {"profile_id": self.profileID, "target_profile_id": targetProfileID})

    isExist           = lambda self, username: makeRubinoData(self.auth, "isExistUsername", {"username": username.replace('@','')})

    like              = lambda self, post_id, post_profile_id: makeRubinoData(self.auth, "likePostAction", {"action_type": "Like", "post_id": post_id, "post_profile_id": post_profile_id, "profile_id": self.profileID})

    reloadProfile     = lambda self                                  : makeRubinoData(self.auth, "updateProfile", {"profile_id": self.profileID, "profile_status": "Public"})
    requestUploadFile = lambda self, file, size=None, Type="Picture" : makeRubinoData(self.auth, "requestUploadFile", {"file_name": file.split("/")[-1], "file_size": size or Path(file).stat().st_size, "file_type": Type, "profile_id": self.profileID})

    unbookmark        = lambda self, postID, postProfileID : makeRubinoData(self.auth, "postBookmarkAction", {"action_type": "Unbookmark","post_id": postID,"post_profile_id": postProfileID,"profile_id": self.profileID})
    unfollow          = lambda self, followee_id: makeRubinoData(self.auth, "requestFollow", {"f_type": "Unfollow", "followee_id": followee_id, "profile_id": self.profileID})
    unlike            = lambda self, post_id, post_profile_id: makeRubinoData(self.auth, "likePostAction", {"action_type": "Unlike", "post_id": post_id, "post_profile_id": post_profile_id, "profile_id": self.profileID})
    updateProfile     = lambda self, name=None, bio=None, email=None: makeRubinoData(self.auth, "updateProfile", {"name": name, "bio": bio, "email": email})
    
    viewPost          = lambda self, post_id, post_profile_id   : makeRubinoData(self.auth, "addPostViewCount", {"post_id": post_id, "post_profile_id": post_profile_id})
    viewStories         = lambda self, storyIds, storyProfileID : makeRubinoData(self.auth, "addViewStory", {"profile_id": self.profileID, "story_ids": storyIds, "story_profile_id": storyProfileID})


from re import findall
from rich import print
from threading import Thread
from datetime import datetime

bot = None
socket = None
lockTime = None
unlockTime = None

class Tools:
    def __init__(self, auth):
        from client import Bot, Socket
        bot = Bot("", auth=auth)
        socket = Socket(auth)

    def hasInsult(self, msg):
        return any(word in open("dontReadMe.txt").read().split("\n") for word in msg.split())
    
    def hasAD(self, msg):
        result = False
        links = list(map(lambda ID: ID.strip()[1:],findall(r"@[\w|_|\d]+", msg))) + list(map(lambda link:link.split("/")[-1],findall(r"rubika\.ir/\w+",msg)))
        joincORjoing = "joing" in msg or "joinc" in msg
    
        if joincORjoing: result = True
        else:
            for link in links:
                try:
                    Type = bot.getInfoByUsername(link)["data"]["chat"]["abs_object"]["type"]
                    if Type == "Channel": result = True
                except KeyError: result = False

        return result

    def hasSpam(self, messages):
        result, beforeIter = {"isExist":False, "spams":[]}, 0
        for Iter in range(0,len(messages)):
            if messages[beforeIter] == messages[Iter] and Iter != beforeIter:
                result["isExist"] = True
                result["spams"].append([beforeIter, Iter])
            beforeIter = Iter
        return result

    class lockSchedule:
        def __init__(self): pass

        def setLockTime(self, h, m):
            global lockTime
            lockTime = f"{h}:{m}"
    
        def setUnlockTime(self, h, m):
            global unlockTime
            unlockTime = f"{h}:{m}"

        def getLockTime(self):
            global lockTime
            return lockTime

        def getUnlockTime(self):
            global unlockTime
            return unlockTime
    
        def checkLockTime(self, guid, accesses=[]):
            global lockTime
            if datetime.now().strftime("%H:%M") == lockTime:
                bot.setMembersAccess(guid, accesses)
    
        def checkUnlockTime(self, guid, accesses=["ViewMembers","ViewAdmins","SendMessages","AddMember"]):
            global lockTime
            if datetime.now().strftime("%H:%M") == lockTime:
                bot.setMembersAccess(guid, accesses)

    def commandHandler(self, message, command, method:callable):
        if message == command: method()

    def isJoin(self, user_guid:str, channel_guid:str) -> bool:
        userInfo       = bot.getInfo(user_guid)
        haveUserName   = 'username' in userInfo.keys()
        channelMembers = lambda keyword : bot.getMembers(channel_guid, search_text=keyword) if keyword != None else []

        if haveUserName: result = [True for i in channelMembers(userInfo.get('username'))     if i['member_guid'] == user_guid][0]
        else:
            result = [True for i in channelMembers(userInfo.get('first_name'))   if i['member_guid'] == user_guid][0]
            result = [True for i in channelMembers(userInfo.get('last_name'))    if i['member_guid'] == user_guid][0]

        return result == True

    def faster(self, controller):
        def get(): socket.handle()
        def use(): controller(socket.data)
        
        while True:
            Thread(target=get).start()
            Thread(target=use).start()

    loadTime = lambda timestamp, func=__import__("datetime").datetime.fromtimestamp: func(int(timestamp)) # func can be __import__("jdatetime").datetime.fromtimestamp THEN you must install jdatetime using `pip install jdatetime`

    @staticmethod
    def chatDetection(chat_id:str) -> str:
        return "Group" if chat_id.startswith("g") else "Channel" if chat_id.startswith("c") else "User" if chat_id.startswith("u") else "Bot" if chat_id.startswith("b") else "Service"

    @staticmethod
    def parse(mode, text):
        results = []
        
        replaces = ["**", "__", "`", "[", "]", "(", ")", "~~", "--"] if mode.lower() == "markdown" else ["<b>", "<i>", "<pre>", "<s>", "<u>", "<link href='", "<a href='", "'>", "</a>", "</link>", "</u>", "</s>", "</pre>", "</i>", "</b>"] # the symbols
        realText = text
        for i in replaces:
            if i in text: realText = realText.replace(i, '')
        bolds    = findall(r"\*\*(.*?)\*\*",text) if mode.lower() == "markdown" else findall("<b>(.*?)</b>",text)
        italics  = findall(r"\_\_(.*?)\_\_",text) if mode.lower() == "markdown" else findall("<i>(.*?)</i>",text)
        monos    = findall(r"\`(.*?)\`",text) if mode.lower() == "markdown" else findall("<pre>(.*?)</pre>",text)
        strikes  = findall(r"\~\~(.*?)\~\~",text) if mode.lower() == "markdown" else findall("<s>(.*?)</s>",text)
        unders   = findall(r"\-\-(.*?)\-\-",text) if mode.lower() == "markdown" else findall("<u>(.*?)</u>",text)
        Mentions = findall(r"\[(.*?)\]\((.*?)\)", text) if mode.lower() == "markdown" else findall("<a href='(.*?)'>(.*?)</a>", text)

        bResult = [realText.index(i) for i in bolds]
        iResult = [realText.index(i) for i in italics]
        mResult = [realText.index(i) for i in monos]
        sResult = [realText.index(i) for i in strikes]
        uResult = [realText.index(i) for i in unders]

        for m in Mentions: realText = realText.replace(m[0], "") # removing guids & links from orginal text
        MResult = [realText.index(i[1]) for i in Mentions]
        
        for indexes,words,Types in ((bResult, bolds, ["Bold"]*len(bolds)), (iResult, italics, ["Italic"]*len(italics)), (mResult, monos, ["Mono"]*len(monos)), (sResult, strikes, ["Strike"]), (uResult, unders, ["Underline"]), (MResult, Mentions, ["MentionText"]*len(Mentions))):
            if indexes != [] and words != []:
                for index, word, Type in zip(indexes, words, Types):
                    result = {"from_index": index,"length": len(word if Type != "MentionText" else word[1]),"type": Type}
                    if Type == "MentionText":
                        if word[0].startswith('http'): result["type"], result["link"] = "Link", {"hyperlink_data": {"url": word[0]}, "type": "hyperlink"}
                        else: result["mention_text_object_guid"], result["mention_text_object_type"] = word[0], Tools.chatDetection(word[0])
                    results.append(result)

        return dict(metadata=results, realText=realText)

    @staticmethod
    def getThumbInline(image_bytes:bytes) -> str:
        import io, base64, PIL.Image
        im, output = PIL.Image.open(io.BytesIO(image_bytes)), io.BytesIO()
        width, height = im.size
        im.save(output, format='PNG')
        im_data = output.getvalue()
        image_data = base64.b64encode(im_data)
        if not isinstance(image_data, str): image_data = image_data.decode()
        return image_data

    @staticmethod
    def getImageSize(image_bytes:bytes) -> list:
        import io, PIL.Image
        im = PIL.Image.open(io.BytesIO(image_bytes))
        width, height = im.size
        return [width , height]


from asyncio  import run
from random   import sample, choice
from json     import loads, dumps, decoder
from datetime import datetime
from aiohttp  import ClientSession, client_exceptions
from string   import ascii_lowercase, ascii_uppercase, digits

from encryption import encryption
from exceptions import *

__version__ , __license__ , __copyright__ = "6.0.6" , "GPLv3 license" , f"Copyright (C) {datetime.now().year} Bahman Ahmadi <github.com/Bahman-Ahmadi>"

class accesses:
    class admin:
        pin               = "PinMessages"
        newAdmin          = "SetAdmin"
        editInfo          = "ChangeInfo"
        banMember         = "BanMember"
        changeLink        = "SetJoinLink"
        editMembersAccess = "SetMemberAccess"
        deleteMessages    = "DeleteGlobalAllMessages"

    class user:
        viewMembers = "ViewMembers"
        viewAdmins  = "ViewAdmins"
        sendMessage = "SendMessages"
        addMember   = "AddMember"

class clients:
    pwa = {
        "app_name"    : "Main",
        "app_version" : "1.2.1",
        "platform"    : "PWA",
        "package"     : "m.rubika.ir",
        "lang_code"   : "fa"
    }
    web = {
        "app_name"    : "Main",
        "app_version" : "4.0.7",
        "platform"    : "Web",
        "package"     : "web.ir",
        "lang_code"   : "fa"
    }
    android = {
        "app_name"    : "Main",
        "app_version" : "2.9.8",
        "platform"    : "Android",
        "package"     : "app.rbmain.a",
        "lang_code"   : "fa"
    }

class reports:
    OTHER       = 100
    VIOLENCE    = 101
    SPAM        = 102
    PORNOGRAPHY = 103
    CHILD_ABUSE = 104
    COPYRIGHT   = 105
    FISHING     = 106

class visibility:
    EVERYBODY  = "Everybody"
    MYCONTACTS = "MyContacts"
    NOBODY     = "Nobody"

randStr = lambda length, choices=[*ascii_lowercase, *ascii_uppercase, *digits, *"-_"]: "".join([choice(choices) for i in range(length)])

defaultDevice = {
    "app_version"      : "MA_2.9.8",
    "device_hash"      : randStr(32, [*ascii_uppercase, *digits]),
    "device_model"     : "rubikalib",
    "is_multi_account" : False,
    "lang_code"        : "fa",
    "system_version"   : "SDK 22",
    "token"            : f"{randStr(22, [*ascii_lowercase, *ascii_uppercase, *digits])}:{randStr(140)}",
    "token_type"       : "Firebase"
}

def post(json:dict, method, url:str=None, platform="rubika", enc:encryption=None, isEncrypted:bool=True) -> dict:
    async def POST(url, data):
        while 1:
            try:
                async with ClientSession() as session:
                    async with session.post(url, json=data) as response:
                        response = await response.text()
                        response = loads(str(enc.decrypt(loads(response).get("data_enc")))) if "data_enc" in loads(response).keys() and isEncrypted else loads(response)
                        if "status" in response.keys() and response.get("status") != "OK":
                            if response.get("status_det") == "NOT_REGISTERED":
                                raise NotRegistered("the auth is incorrect. please sure about your account's health then login again.")
                            elif response.get("status_det") == "INVALID_INPUT" :
                                print(url)
                                raise InvalidInput(f"the inserted argument(s) is invaild in the {platform}/{method}. if you're sure about your argument(s), please report this message.")
                            elif response.get("status_det") == "TOO_REQUESTS" :
                                raise TooRequests(f"the {platform}/{method} method has been limited. please try again later.")
                            elif response.get("status_det") == 'INVALID_AUTH':
                                raise InvaildAuth(f"the inserted argument(s) in {platform}/{method} is vaild but is not related to other argument(s) or maybe for other reasons, anyway now this method can't run on server. please don't enter fake argument(s) and fix anything can return this exception")
                        else:
                            return response
            except decoder.JSONDecodeError: ...
            except client_exceptions.ClientConnectorError: url = _getURL(dc_id=64)
    return run(POST(url or _getURL(), json))

retries = 0
def _getURL(key="default_api_urls", DCsURL:str="https://messengerg2cX.iranlms.ir/", getDCsURL:str="https://getdcmess.iranlms.ir", dc_id:int=None):
    import requests
    while 1:
        try:
            res = requests.post(json={"api_version": 4, "client": clients.pwa, "method": "getDCs"}, url=getDCsURL).json().get("data").get(key)
            return DCsURL.replace('X', dc_id) if dc_id is not None else choice(list(res))
        except requests.exceptions.ConnectionError:
            retries += 1
            if retries == 3:
                retries = 0
                break

def makeData(api_version:int, auth:str, method:str, data:dict, client:dict, url:str=None) -> dict :
    url, enc = url or _getURL(), encryption(auth)
    outerJson = {
        "api_version" : str(api_version),
        "auth"        : auth,
        "data_enc"    : enc.encrypt(dumps({
            "method"  : method,
            "input"   : data,
            "client"  : client
        }))
    }

    if int(api_version) == 4:
        outerJson["data_enc"] = enc.encrypt(dumps(data))
        outerJson["client"]   = client
        outerJson["method"]   = method

    return post(outerJson, url=url, platform=client.get('package'), method=method, enc=enc)

def makeTmpData(method:str, data:dict, url:str=None) -> dict:
    url, tmp = url or _getURL(), tmpGeneration()
    enc = encryption(tmp)
    outerJson = {
        "api_version": "5",
        "tmp_session": tmp,
        "data_enc"   : enc.encrypt(dumps({
            "method" : method,
            "input"  : data,
            "client" : clients.web
        }))
    }
    
    return post(outerJson, method, url=url, platform=clients.web.get("package"), enc=enc)

def makeRubinoData(auth:str, method:str, data:dict) -> dict:
    outerJson = {
        "api_version": "0",
        "auth"   : auth,
        "client" : clients.android,
        "data"   : data,
        "method" : method
    }
    return post(outerJson, method, url="https://rubino12.iranlms.ir/", platform="rubino", isEncrypted=False)

tmpGeneration = lambda: randStr(32, [*ascii_lowercase, *digits])

def welcome(text, time:float=0.035):
    from time import sleep
    try:
        from rich import print as Print
        Print(text)
    except ModuleNotFoundError:
        for char in text:
            print(char, end='', flush=True)
            sleep(float(time))
    print()
